@@ -43,10 +43,10 @@
 //! # Helper crates we use
 //!
 //! * `cssparser` crate as a CSS tokenizer, and some utilities to
-//! parse CSS rules and declarations.
+//!   parse CSS rules and declarations.
 //!
 //! * `selectors` crate for the representation of selectors and
-//! selector lists, and for the matching engine.
+//!   selector lists, and for the matching engine.
 //!
 //! Both crates provide very generic implementations of their concepts,
 //! and expect the caller to provide implementations of various traits,
@@ -56,10 +56,10 @@
 //! the following types:
 //!
 //! * A parsed CSS rule.  For `fill: blue;` we have
-//! `ParsedProperty::Fill(...)`.
+//!   `ParsedProperty::Fill(...)`.
 //!
 //! * A parsed selector list; we use `SelectorList` from the
-//! `selectors` crate.
+//!   `selectors` crate.
 //!
 //! In turn, the `selectors` crate needs a way to navigate and examine
 //! one's implementation of an element tree.  We provide `impl
@@ -74,13 +74,13 @@
 //! matches by specificity and apply the result to each element.
 
 use cssparser::{
-    match_ignore_ascii_case, parse_important, AtRuleParser, BasicParseErrorKind, CowRcStr,
+    self, match_ignore_ascii_case, parse_important, AtRuleParser, BasicParseErrorKind, CowRcStr,
     DeclarationParser, Parser, ParserInput, ParserState, QualifiedRuleParser, RuleBodyItemParser,
     RuleBodyParser, SourceLocation, StyleSheetParser, ToCss,
 };
 use data_url::mime::Mime;
 use language_tags::LanguageTag;
-use markup5ever::{namespace_url, ns, Namespace, QualName};
+use markup5ever::{self, namespace_url, ns, Namespace, QualName};
 use selectors::attr::{AttrSelectorOperation, CaseSensitivity, NamespaceConstraint};
 use selectors::matching::{
     ElementSelectorFlags, IgnoreNthChildForInvalidation, MatchingContext, MatchingMode,
@@ -93,6 +93,7 @@ use std::fmt;
 use std::str;
 use std::str::FromStr;
 
+use crate::element::Element;
 use crate::error::*;
 use crate::io::{self, BinaryData};
 use crate::node::{Node, NodeBorrow, NodeCascade};
@@ -185,6 +186,7 @@ pub struct RuleParser {
 }
 
 /// Errors from the CSS parsing process
+#[allow(dead_code)] // looks like we are not actually using this yet?
 #[derive(Debug)]
 pub enum ParseErrorKind<'i> {
     Selector(selectors::parser::SelectorParseErrorKind<'i>),
@@ -571,7 +573,17 @@ impl selectors::Element for RsvgElement {
 
     /// Converts self into an opaque representation.
     fn opaque(&self) -> OpaqueElement {
-        OpaqueElement::new(&self.0.borrow())
+        // The `selectors` crate uses this value just for pointer comparisons, to answer
+        // the question, "is this element the same as that one?".  So, we'll give it a
+        // reference to our actual node's data, i.e. skip over whatever wrappers there
+        // are in rctree.
+        //
+        // We use an explicit type here to make it clear what the type is; otherwise you
+        // may be fooled by the fact that borrow_element() returns a Ref<Element>, not a
+        // plain reference: &Ref<T> is transient and would get dropped at the end of this
+        // function, but we want something long-lived.
+        let element: &Element = &self.0.borrow_element();
+        OpaqueElement::new::<Element>(element)
     }
 
     fn parent_element(&self) -> Option<Self> {
@@ -686,7 +698,11 @@ where {
                 .xml_lang()
                 .0
                 .as_ref()
-                .map_or(false, |e_lang| css_lang.iter().any(|l| l.matches(e_lang))),
+                .map_or(false, |e_lang| {
+                    css_lang
+                        .iter()
+                        .any(|l| l.is_language_range() && l.matches(e_lang))
+                }),
         }
     }
 
